@@ -7,11 +7,12 @@ import (
 	"strings"
 	"time"
 
+	"github.com/golang/protobuf/ptypes"
 	"github.com/hashicorp/go-hclog"
 	"github.com/mitchellh/mapstructure"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
-	"k8s.io/apimachinery/pkg/api/resource"
+	k8sresource "k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/apimachinery/pkg/util/wait"
@@ -19,8 +20,11 @@ import (
 
 	"github.com/hashicorp/waypoint-plugin-sdk/component"
 	"github.com/hashicorp/waypoint-plugin-sdk/docs"
+	"github.com/hashicorp/waypoint-plugin-sdk/framework/resource"
+	sdk "github.com/hashicorp/waypoint-plugin-sdk/proto/gen"
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
 	"github.com/hashicorp/waypoint/builtin/docker"
+	"github.com/hashicorp/waypoint/internal/clierrors"
 )
 
 const (
@@ -69,6 +73,10 @@ func (p *Platform) ValidateAuth() error {
 	return nil
 }
 
+func (p *Platform) StatusFunc() interface{} {
+	return p.Status
+}
+
 // DefaultReleaserFunc implements component.PlatformReleaser
 func (p *Platform) DefaultReleaserFunc() interface{} {
 	var rc ReleaserConfig
@@ -84,44 +92,64 @@ func (p *Platform) DefaultReleaserFunc() interface{} {
 	}
 }
 
-// Deploy deploys an image to Kubernetes.
-func (p *Platform) Deploy(
-	ctx context.Context,
-	log hclog.Logger,
-	src *component.Source,
-	img *docker.Image,
-	deployConfig *component.DeploymentConfig,
-	ui terminal.UI,
-) (*Deployment, error) {
-	// Create our deployment and set an initial ID
-	var result Deployment
-	id, err := component.Id()
-	if err != nil {
-		return nil, err
-	}
-	result.Id = id
-	result.Name = strings.ToLower(fmt.Sprintf("%s-%s", src.App, id))
-
-	sg := ui.StepGroup()
-	step := sg.Add("Initializing Kubernetes client...")
-	defer step.Abort()
+func (p *Platform) resourceManager(log hclog.Logger) *resource.Manager {
+	return resource.NewManager(
+		resource.WithLogger(log.Named("resource_manager")),
+		resource.WithValueProvider(p.getClientset),
+		resource.WithResource(resource.NewResource(
+			resource.WithName("deployment"),
+			resource.WithState(&Resource_Deployment{}),
+			resource.WithCreate(p.resourceDeploymentCreate),
+			resource.WithDestroy(p.resourceDeploymentDestroy),
+		)),
+	)
+	return nil
+}
 
+// getClientset is a value provider for our resource manager and provides
+// the connection information used by resources to interact with Kubernetes.
+func (p *Platform) getClientset() (*clientsetInfo, error) {
 	// Get our client
 	clientSet, ns, config, err := clientset(p.config.KubeconfigPath, p.config.Context)
 	if err != nil {
 		return nil, err
 	}
 
-	// Override namespace if set
+	return &clientsetInfo{
+		Clientset: clientSet,
+		Namespace: ns,
+		Config:    config,
+	}, nil
+}
+
+// resourceDeploymentCreate creates the Kubernetes deployment.
+func (p *Platform) resourceDeploymentCreate(
+	ctx context.Context,
+	log hclog.Logger,
+	src *component.Source,
+	img *docker.Image,
+	deployConfig *component.DeploymentConfig,
+	ui terminal.UI,
+
+	result *Deployment,
+	state *Resource_Deployment,
+	csinfo *clientsetInfo,
+	sg terminal.StepGroup,
+) error {
+	// Prepare our namespace and override if set.
+	ns := csinfo.Namespace
 	if p.config.Namespace != "" {
 		ns = p.config.Namespace
 	}
 
-	step.Update("Kubernetes client connected to %s with namespace %s", config.Host, ns)
+	step := sg.Add("")
+	defer func() { step.Abort() }()
+	step.Update("Kubernetes client connected to %s with namespace %s", csinfo.Config.Host, ns)
 	step.Done()
 
 	step = sg.Add("Preparing deployment...")
 
+	clientSet := csinfo.Clientset
 	deployClient := clientSet.AppsV1().Deployments(ns)
 
 	// Determine if we have a deployment that we manage already
@@ -133,9 +161,10 @@ func (p *Platform) Deploy(
 		err = nil
 	}
 	if err != nil {
-		return nil, err
+		return err
 	}
 
+	// Setup our port configuration
 	if p.config.ServicePort == 0 && p.config.Ports == nil {
 		// nothing defined, set up the defaults
 		p.config.Ports = make([]map[string]string, 1)
@@ -146,7 +175,7 @@ func (p *Platform) Deploy(
 		p.config.Ports[0] = map[string]string{"port": strconv.Itoa(int(p.config.ServicePort)), "name": "http"}
 	} else if p.config.ServicePort > 0 && len(p.config.Ports) > 0 {
 		// both defined, this is an error
-		return nil, fmt.Errorf("Cannot define both 'service_port' and 'ports'. Use" +
+		return fmt.Errorf("Cannot define both 'service_port' and 'ports'. Use" +
 			" 'ports' for configuring multiple container ports.")
 	}
 
@@ -157,14 +186,12 @@ func (p *Platform) Deploy(
 			Value: fmt.Sprint(p.config.Ports[0]["port"]),
 		},
 	}
-
 	for k, v := range p.config.StaticEnvVars {
 		env = append(env, corev1.EnvVar{
 			Name:  k,
 			Value: v,
 		})
 	}
-
 	for k, v := range deployConfig.Env() {
 		env = append(env, corev1.EnvVar{
 			Name:  k,
@@ -182,6 +209,7 @@ func (p *Platform) Deploy(
 	// Set our ID on the label. We use this ID so that we can have a key
 	// to route to multiple versions during release management.
 	deployment.Spec.Template.Labels[labelId] = result.Id
+
 	// Version label duplicates "labelId" to support services like Istio that
 	// expect pods to be labled with 'version'
 	deployment.Spec.Template.Labels["version"] = result.Id
@@ -199,26 +227,26 @@ func (p *Platform) Deploy(
 	}
 
 	// Get container resource limits and requests
-	var resourceLimits = make(map[corev1.ResourceName]resource.Quantity)
-	var resourceRequests = make(map[corev1.ResourceName]resource.Quantity)
+	var resourceLimits = make(map[corev1.ResourceName]k8sresource.Quantity)
+	var resourceRequests = make(map[corev1.ResourceName]k8sresource.Quantity)
 
 	for k, v := range p.config.Resources {
 		if strings.HasPrefix(k, "limits_") {
 			limitKey := strings.Split(k, "_")
 			resourceName := corev1.ResourceName(limitKey[1])
 
-			quantity, err := resource.ParseQuantity(v)
+			quantity, err := k8sresource.ParseQuantity(v)
 			if err != nil {
-				return nil, err
+				return err
 			}
 			resourceLimits[resourceName] = quantity
 		} else if strings.HasPrefix(k, "requests_") {
 			reqKey := strings.Split(k, "_")
 			resourceName := corev1.ResourceName(reqKey[1])
 
-			quantity, err := resource.ParseQuantity(v)
+			quantity, err := k8sresource.ParseQuantity(v)
 			if err != nil {
-				return nil, err
+				return err
 			}
 			resourceRequests[resourceName] = quantity
 		} else {
@@ -404,13 +432,13 @@ func (p *Platform) Deploy(
 			err = nil
 		}
 		if err != nil {
-			return nil, err
+			return err
 		}
 
 		if saCreate {
 			serviceAccount, err = saClient.Create(ctx, serviceAccount, metav1.CreateOptions{})
 			if err != nil {
-				return nil, err
+				return err
 			}
 		}
 	}
@@ -428,9 +456,13 @@ func (p *Platform) Deploy(
 		deployment, err = dc.Update(ctx, deployment, metav1.UpdateOptions{})
 	}
 	if err != nil {
-		return nil, err
+		return err
 	}
 
+	// We successfully created or updated, so set the name on our state so
+	// that if we error, we'll partially clean up properly. THIS IS IMPORTANT.
+	state.Name = result.Name
+
 	step.Done()
 	step = sg.Add("Waiting for deployment...")
 
@@ -509,12 +541,81 @@ func (p *Platform) Deploy(
 		if err == wait.ErrWaitTimeout {
 			err = fmt.Errorf("Deployment was not able to start pods after %s", timeout)
 		}
-		return nil, err
+		return err
 	}
 
 	step.Update("Deployment successfully rolled out!")
 	step.Done()
 
+	return nil
+}
+
+// Destroy deletes the K8S deployment.
+func (p *Platform) resourceDeploymentDestroy(
+	ctx context.Context,
+	state *Resource_Deployment,
+	sg terminal.StepGroup,
+	csinfo *clientsetInfo,
+) error {
+	// Prepare our namespace and override if set.
+	ns := csinfo.Namespace
+	if p.config.Namespace != "" {
+		ns = p.config.Namespace
+	}
+
+	step := sg.Add("")
+	defer func() { step.Abort() }()
+	step.Update("Kubernetes client connected to %s with namespace %s", csinfo.Config.Host, ns)
+	step.Done()
+
+	step = sg.Add("Deleting deployment...")
+	deployclient := csinfo.Clientset.AppsV1().Deployments(ns)
+	if err := deployclient.Delete(ctx, state.Name, metav1.DeleteOptions{}); err != nil {
+		return err
+	}
+	step.Done()
+
+	return nil
+}
+
+// Deploy deploys an image to Kubernetes.
+func (p *Platform) Deploy(
+	ctx context.Context,
+	log hclog.Logger,
+	src *component.Source,
+	img *docker.Image,
+	deployConfig *component.DeploymentConfig,
+	ui terminal.UI,
+) (*Deployment, error) {
+	// Create our deployment and set an initial ID
+	var result Deployment
+	id, err := component.Id()
+	if err != nil {
+		return nil, err
+	}
+	result.Id = id
+	result.Name = strings.ToLower(fmt.Sprintf("%s-%s", src.App, id))
+
+	// We'll update the user in real time
+	sg := ui.StepGroup()
+	defer sg.Wait()
+
+	if p.config.ServicePort == 0 {
+		p.config.ServicePort = 3000
+	}
+
+	// Create our resource manager and create
+	rm := p.resourceManager(log)
+	if err := rm.CreateAll(
+		ctx, log, sg, ui,
+		src, img, deployConfig, &result,
+	); err != nil {
+		return nil, err
+	}
+
+	// Store our resource state
+	result.ResourceState = rm.State()
+
 	return &result, nil
 }
 
@@ -526,31 +627,99 @@ func (p *Platform) Destroy(
 	ui terminal.UI,
 ) error {
 	sg := ui.StepGroup()
-	step := sg.Add("Initializing Kubernetes client...")
-	defer step.Abort()
+	defer sg.Wait()
 
-	// Get our client
-	clientset, ns, config, err := clientset(p.config.KubeconfigPath, p.config.Context)
+	rm := p.resourceManager(log)
+
+	// If we don't have resource state, this state is from an older version
+	// and we need to manually recreate it.
+	if deployment.ResourceState == nil {
+		rm.Resource("deployment").SetState(&Resource_Deployment{
+			Name: deployment.Name,
+		})
+	} else {
+		// Load our set state
+		if err := rm.LoadState(deployment.ResourceState); err != nil {
+			return err
+		}
+	}
+
+	// Destroy
+	return rm.DestroyAll(ctx, log, sg, ui)
+}
+
+func (p *Platform) Status(
+	ctx context.Context,
+	log hclog.Logger,
+	deployment *Deployment,
+	ui terminal.UI,
+) (*sdk.StatusReport, error) {
+	sg := ui.StepGroup()
+	defer sg.Wait()
+
+	step := sg.Add("Gathering health report for Kubernetes platform...")
+	defer func() { step.Abort() }() // Defer in func in case more steps are added to this func in the future
+
+	csInfo, err := p.getClientset()
 	if err != nil {
-		return err
+		return nil, err
 	}
-
-	// Override namespace if set
+	clientSet := csInfo.Clientset
+	namespace := csInfo.Namespace
 	if p.config.Namespace != "" {
-		ns = p.config.Namespace
+		namespace = p.config.Namespace
 	}
 
-	step.Update("Kubernetes client connected to %s with namespace %s", config.Host, ns)
-	step.Done()
-	step = sg.Add("Deleting deployment...")
-
-	deployclient := clientset.AppsV1().Deployments(ns)
-	if err := deployclient.Delete(ctx, deployment.Name, metav1.DeleteOptions{}); err != nil {
-		return err
+	podClient := clientSet.CoreV1().Pods(namespace)
+	podLabelId := fmt.Sprintf("app=%s", deployment.Name)
+	podList, err := podClient.List(ctx, metav1.ListOptions{LabelSelector: podLabelId})
+	if err != nil {
+		ui.Output(
+			"Error listing pods to determine application health: %s", clierrors.Humanize(err),
+			terminal.WithErrorStyle(),
+		)
+		return nil, err
 	}
 
+	// Create our status report
+	step.Update("Building status report for running pods...")
+	result := buildStatusReport(podList)
+
+	result.TimeGenerated = ptypes.TimestampNow()
+	log.Debug("status report complete")
+
+	// update output based on main health state
+	step.Update("Finished building report for Kubernetes platform")
 	step.Done()
-	return nil
+
+	// NOTE(briancain): Replace ui.Status with StepGroups once this bug
+	// has been fixed: https://github.com/hashicorp/waypoint/issues/1536
+	st := ui.Status()
+	defer st.Close()
+
+	st.Update("Determining overall container health...")
+	if result.Health == sdk.StatusReport_READY {
+		st.Step(terminal.StatusOK, fmt.Sprintf("Deployment %q is reporting ready!", deployment.Name))
+	} else {
+		if result.Health == sdk.StatusReport_PARTIAL {
+			st.Step(terminal.StatusWarn, fmt.Sprintf("Deployment %q is reporting partially available!", deployment.Name))
+		} else {
+			st.Step(terminal.StatusError, fmt.Sprintf("Deployment %q is reporting not ready!", deployment.Name))
+		}
+
+		// Extra advisory wording to let user know that the deployment could be still starting up
+		// if the report was generated immediately after it was deployed or released.
+		st.Step(terminal.StatusWarn, mixedHealthWarn)
+	}
+
+	// More UI detail for non-ready resources
+	for _, resource := range result.Resources {
+		if resource.Health != sdk.StatusReport_READY {
+			st.Step(terminal.StatusWarn, fmt.Sprintf("Resource %q is reporting %q", resource.Name, resource.Health.String()))
+		}
+	}
+
+	return &result, nil
 }
 
 // Config is the configuration structure for the Platform.
@@ -867,10 +1036,18 @@ deploy "kubernetes" {
 	return doc, nil
 }
 
+var (
+	mixedHealthWarn = strings.TrimSpace(`
+Waypoint detected that the current deployment is not ready, however your application
+might be available or still starting up.
+`)
+)
+
 var (
 	_ component.Platform         = (*Platform)(nil)
 	_ component.PlatformReleaser = (*Platform)(nil)
 	_ component.Configurable     = (*Platform)(nil)
 	_ component.Documented       = (*Platform)(nil)
 	_ component.Destroyer        = (*Platform)(nil)
+	_ component.Status           = (*Platform)(nil)
 )
