@@ -3,7 +3,6 @@ import { currentURL } from '@ember/test-helpers';
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
 import { visitable, create, collection } from 'ember-cli-page-object';
-import a11yAudit from 'ember-a11y-testing/test-support/audit';
 import login from 'waypoint/tests/helpers/login';
 
 const url = '/default/microchip/app/wp-bandwidth/releases';
@@ -19,8 +18,11 @@ module('Acceptance | releases list', function (hooks) {
   login();
 
   test('visiting releases page', async function (assert) {
+    let project = this.server.create('project', { name: 'microchip' });
+    let application = this.server.create('application', { name: 'wp-bandwidth', project });
+    this.server.createList('release', 3, { application });
+
     await page.visit();
-    await a11yAudit();
 
     assert.equal(page.list.length, 3);
     assert.equal(currentURL(), url);
