@@ -34,12 +34,14 @@ export default function App({ Component, pageProps }) {
         siteName={title}
         description={description}
         image="https://www.waypointproject.io/img/og-image.png"
-        icon={[{ href: '/favicon.ico' }]}
+        icon={[{ href: '/_favicon.ico' }]}
       >
         <meta name="og:title" property="og:title" content={title} />
         <meta name="og:description" property="og:title" content={description} />
       </HashiHead>
-      {ALERT_BANNER_ACTIVE && <AlertBanner {...alertBannerData} theme="blue" />}
+      {ALERT_BANNER_ACTIVE && (
+        <AlertBanner {...alertBannerData} product="waypoint" />
+      )}
       <HashiStackMenu />
       <ProductSubnav />
       <div className="content">
